;***********************************************************************************
; Based on 8051 C code from MST, (c) Sensirion AG
; only main functions have been tested
;***********************************************************************************/
; SLO HAN SIDO PROBADAS LAS FUNCIONES PRINCIPALES (las descriptas en CAN-047)

		list	p=16F630           ; list directive to define processor
        #include	<p16F630.inc>
        errorlevel -302			; suppress "register not in bank0, check page bits" message.	
;
		RADIX	dec

; DAT must be LSB, otherwise, code must be modified
#define DAT	PORTC,0
#define DATPORT PORTC
#define DATT	TRISC,0
#define SCK		PORTC,1


#define SHT_TMOUT 2000

;                            addr  command  r/w
#define STATUS_REG_W 0x06   ; 000   0011    0
#define STATUS_REG_R 0x07   ; 000   0011    1
#define MEASURE_TEMP 0x03   ; 000   0001    1
#define MEASURE_HUMI 0x05   ; 000   0010    1
#define SHT_RESET    0x1e   ; 000   1111    0

shtvars	UDATA_SHR
dat		RES 1
bitcntr	RES 1
value	RES	2
chksum	RES 1
flags	RES 1

#define ACK flags,0


		GLOBAL value,chksum


shtcode	CODE
; Send  a byte
; Call with data in W
; Returns status in C flag
; C -> Error
s_write_byte:
		movwf dat				; Store data
		movlw 8					; 8 bits
		movwf bitcntr
swbl1	rlf dat,f				; C = MSB (shift bit for output)
		bcf SCK					; CLK = 0
		nop
		bcf DAT					; DATA = 0 (open-collector)
		bsf STATUS,RP0 			; Bank 1
		bsf DATT				; pull-up
		btfss STATUS,C			; if C, DATA = pull-up
		bcf DATT				; otherwise DATA = 0
		bcf STATUS,RP0 			; Bank 0
		bsf SCK					; CLK = 1
		decfsz bitcntr,f
		goto swbl1
		bcf SCK					; CLK = 0
		bsf STATUS,RP0 			; Bank 1
		bsf DATT				; pull-up
		bcf STATUS,RP0 			; Bank 0
		bsf SCK					; CLK = 1 (9th clk)
		rrf DATPORT,W			; check ack (DATA will be pulled down by SHT11)
		bcf SCK					; CLK = 0
		return

; Receive a byte
; Returns data in W
s_read_byte:
		clrf dat				; DATA = 0
		movlw 8					; 8 bits
		movwf bitcntr
srbl1	bsf SCK					; CLK = 1
		nop
		rrf DATPORT,W			; read data in C
		rlf dat,f				; and into register	(MSB first)	
		bcf SCK					; CLK = 0
		decfsz bitcntr,f
		goto srbl1
		bcf DAT					; DATA = 0 (open-collector)
		bsf STATUS,RP0 			; Bank 1
		btfsc ACK				; if need to ACK
		bcf DATT				; DATA=0 (ACK)
		bcf STATUS,RP0 			; Bank 0
		bsf SCK					; CLK = 1 (9th clk)
		nop
		bcf SCK					; CLK = 0
		bsf STATUS,RP0 			; Bank 1
		bsf DATT				; pull-up
		bcf STATUS,RP0 			; Bank 0
		movf dat,W
		return


; communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
;       _____________________________________________________         ________
; DATA:                                                      |_______|
;          _    _    _    _    _    _    _    _    _        ___     ___
; SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______
; Call first time, or when you sense errors
s_connectionreset:
		bcf SCK					; CLK = 0
		movlw 9					; 9 SCK cycles
		movwf bitcntr
scrl1	bsf SCK					; CLK = 1 
		nop
		bcf SCK					; CLK = 0
		decfsz bitcntr,f
		goto scrl1
  				                 ; follow through transmission start
;generates a transmission start
;       _____         ________
; DATA:      |_______|
;           ___     ___
; SCK : ___|   |___|   |______
s_transstart:
		bcf SCK					; CLK = 0
		nop
		bsf SCK					; CLK = 1
		nop
		bcf DAT					; DATA = 0 (open-collector)
		bsf STATUS,RP0 			; Bank 1
		bcf DATT				; DATA=0
		bcf STATUS,RP0 			; Bank 0
		bcf SCK					; CLK = 0
		nop
		bsf SCK					; CLK = 1
		bsf STATUS,RP0 			; Bank 1
		bsf DATT				; pull-up
		bcf STATUS,RP0 			; Bank 0
		bcf SCK					; CLK = 0
		return

s_softreset:
; resets the sensor by a softreset
; Returns write_byte error
		call s_connectionreset	; reset communication
		movlw SHT_RESET
   		goto s_write_byte		; send RESET-command to sensor

; reads the status register with checksum (8-bit)
; Returns write_byte error
s_read_statusreg:
		call s_transstart		;  transmission start
		movlw STATUS_REG_R
		call s_write_byte		; send command to sensor
		btfsc STATUS,C			; return on error
		return
		bsf ACK					; ACK
		call s_read_byte		; read status register (8-bit)
		movwf value
		bcf ACK					; no ACK
  		call s_read_byte		; read checksum (8-bit)
		movwf chksum
		return

; writes the status register with checksum (8-bit)
; Returns write_byte error
s_write_statusreg:
		call s_transstart		; transmission start
		movlw STATUS_REG_W
		call s_write_byte		; send command to sensor
		btfsc STATUS,C			; return on error
		return
		movf value,W
		goto s_write_byte		; send value of status register

; makes a measurement (temperature) with checksum
s_measure_t:
		call s_transstart		; transmission start
		movlw MEASURE_TEMP
		goto s_write_byte		; send command to sensor

; makes a measurement (humidity) with checksum
s_measure_rh:
		call s_transstart		; transmission start
		movlw MEASURE_HUMI
		goto s_write_byte		; send command to sensor

; Call until it returns C reset or you get tired, whatever comes first
s_measure_wait:
		rrf DATPORT,W			; read data in C
		return

; Get measurement results out of sensor
s_measure_get:
		bsf ACK					; ACK
		call s_read_byte		; read the first byte (MSB)
		movwf value+1
		call s_read_byte		; read the second byte (LSB)
		movwf value
		bcf ACK					; no ACK
  		call s_read_byte		; read checksum (8-bit)
		movwf chksum
		return

		GLOBAL s_connectionreset,s_measure_t,s_measure_rh,s_measure_wait
		GLOBAL s_measure_get,s_read_statusreg,s_write_statusreg,s_softreset

		END
